@@ -29,7 +29,7 @@ import (
 )
 
 const (
-	instanceType              = "m5n.large"
+	defaultInstanceType       = "m5n.large"
 	accessKeyIDSecretKey      = "aws_access_key_id"
 	accessKeySecretKey        = "aws_secret_access_key"
 	internalELBLabel          = "kubernetes.io/role/internal-elb"
@@ -47,6 +47,7 @@ type machineSetConfig struct {
 	Region            string
 	SecurityGroupName string
 	SubnetName        string
+	NATTPort          int64
 }
 
 type awsProvider struct {
@@ -58,13 +59,17 @@ type awsProvider struct {
 	ikePort       int64
 	nattPort      int64
 	clusterName   string
+	instanceType  string
+	gateways      int
 }
 
 func NewAWSProvider(
 	kubeClient kubernetes.Interface, workClient workclient.Interface,
 	eventRecorder events.Recorder,
 	region, infraId, clusterName, credentialsSecretName string,
-	ikePort, nattPort int) (*awsProvider, error) {
+	instanceType string,
+	ikePort, nattPort int,
+	gateways int) (*awsProvider, error) {
 	if region == "" {
 		return nil, fmt.Errorf("cluster region is empty")
 	}
@@ -73,6 +78,10 @@ func NewAWSProvider(
 		return nil, fmt.Errorf("cluster infraId is empty")
 	}
 
+	if gateways < 1 {
+		return nil, fmt.Errorf("the count of gateways is less than 1")
+	}
+
 	if ikePort == 0 {
 		ikePort = helpers.SubmarinerIKEPort
 	}
@@ -81,6 +90,10 @@ func NewAWSProvider(
 		nattPort = helpers.SubmarinerNatTPort
 	}
 
+	if instanceType == "" {
+		instanceType = defaultInstanceType
+	}
+
 	awsClient, err := client.NewClient(kubeClient, clusterName, credentialsSecretName, region)
 	if err != nil {
 		return nil, err
@@ -95,6 +108,8 @@ func NewAWSProvider(
 		ikePort:       int64(ikePort),
 		nattPort:      int64(nattPort),
 		clusterName:   clusterName,
+		instanceType:  instanceType,
+		gateways:      gateways,
 	}, nil
 }
 
@@ -132,33 +147,30 @@ func (a *awsProvider) PrepareSubmarinerClusterEnv() error {
 	if err := a.openPort(masterSecurityGroup, workerSecurityGroup, helpers.SubmarinerRoutePort, "udp"); err != nil {
 		return fmt.Errorf("failed to open route port in security group: %v \n", err)
 	}
-	a.eventRecorder.Eventf("SubmarinerRoutePortOpened", "the submariner route port is opened on aws")
 
 	// Open submariner metrics port (8080/TCP) between all master and worker nodes
 	if err := a.openPort(masterSecurityGroup, workerSecurityGroup, helpers.SubmarinerMetricsPort, "tcp"); err != nil {
 		return fmt.Errorf("failed to open route port in security group: %v \n", err)
 	}
-	a.eventRecorder.Eventf("SubmarinerMetricsPortOpened", "the submariner metrics port is opened on aws")
 
 	// Open IPsec ports (by default, 4500/UDP and 500/UDP) for submariner gateway instances
 	if err := a.openIPsecPorts(vpc); err != nil {
 		return fmt.Errorf("failed to create security group with infraID %s and vpcID %s: %v \n", a.infraId, *vpc.VpcId, err)
 	}
-	a.eventRecorder.Eventf("SubmarinerIPsecPortsOpened", "the submariner IPsec ports are opened on aws")
 
 	// Tag one subnet with label kubernetes.io/role/internal-elb for automatic subnet discovery by aws load balancers or
 	// ingress controllers
-	subnet, err := a.tagSubnet(vpc)
+	subnets, err := a.tagSubnets(vpc)
 	if err != nil {
 		return fmt.Errorf("failed to tag subnet with infraID %s and vpcID %s: %v \n", a.infraId, *vpc.VpcId, err)
 	}
-	a.eventRecorder.Eventf("SubmarinerSubnetTagged", "the subnet %s is tagged on aws", *subnet.SubnetId)
 
 	// Apply a manifest work to create a MachineSet on managed cluster to create a new aws instance for submariner gateway
-	if err := a.deployGatewayNode(*subnet.AvailabilityZone, amiId); err != nil {
+	if err := a.deployGatewayNode(amiId, subnets); err != nil {
 		return fmt.Errorf("failed to create MachineSet for %s: %v \n", a.infraId, err)
 	}
-	a.eventRecorder.Eventf("SubmarinerGatewayManifestworkCreated", "the submariner gateway manifestwork %s is created", workName)
+
+	a.eventRecorder.Eventf("SubmarinerClusterEnvBuild", "the submariner cluster env is build on aws")
 
 	return nil
 }
@@ -175,8 +187,6 @@ func (a *awsProvider) CleanUpSubmarinerClusterEnv() error {
 	// delete the applied machineset manifest work to delete the gateway instance from managed cluster
 	if err := a.deleteGatewayNode(); err != nil {
 		errs = append(errs, fmt.Errorf("failed to delete gateway node for %s: %v \n", a.infraId, err))
-	} else {
-		a.eventRecorder.Eventf("SubmarinerGatewayManifestworkDeleted", "the submariner gateway manifestwork %s is deleted", workName)
 	}
 
 	vpc, err := a.findVPC()
@@ -189,15 +199,11 @@ func (a *awsProvider) CleanUpSubmarinerClusterEnv() error {
 	// untag the subnet that was tagged on preparation phase
 	if err := a.untagSubnet(vpc); err != nil {
 		errs = append(errs, fmt.Errorf("failed to untag subnet for %s: %v \n", a.infraId, err))
-	} else {
-		a.eventRecorder.Eventf("SubmarinerSubnetUnTagged", "the subnet is untagged on aws")
 	}
 
 	// the ipsec sg may has references, result in the sg cannot be deleted, so we revoke ports here
 	if err := a.revokeIPsecPorts(vpc); err != nil {
 		errs = append(errs, fmt.Errorf("failed to revoke ipsec ports for %s: %v \n", a.infraId, err))
-	} else {
-		a.eventRecorder.Eventf("SubmarinerIPsecPortsClosed", "the submariner IPsec ports are closed on aws")
 	}
 
 	// cannot find the worker security group, the below tasks will not continue, return directly
@@ -217,15 +223,15 @@ func (a *awsProvider) CleanUpSubmarinerClusterEnv() error {
 	// revoke Submariner metrics ports
 	if err := a.revokePort(masterSecurityGroup, workerSecurityGroup, helpers.SubmarinerMetricsPort, "tcp"); err != nil {
 		errs = append(errs, fmt.Errorf("failed to revoke metrics port for %s: %v \n", a.infraId, err))
-	} else {
-		a.eventRecorder.Eventf("SubmarinerMetricsPortClosed", "the submariner metrics port is closed on aws")
 	}
 
 	// revoke Submariner route ports
 	if err := a.revokePort(masterSecurityGroup, workerSecurityGroup, helpers.SubmarinerRoutePort, "udp"); err != nil {
 		errs = append(errs, fmt.Errorf("failed to revoke route port for %s: %v \n", a.infraId, err))
-	} else {
-		a.eventRecorder.Eventf("SubmarinerRoutePortClosed", "the submariner route port is closed on aws")
+	}
+
+	if len(errs) == 0 {
+		a.eventRecorder.Eventf("SubmarinerClusterEnvCleanedUp", "the submariner cluster env is cleaned up on aws")
 	}
 
 	return operatorhelpers.NewMultiLineAggregate(errs)
@@ -324,7 +330,7 @@ func (a *awsProvider) findSecurityGroup(vpcId, nameTag string) (*ec2.SecurityGro
 	return securityGroups.SecurityGroups[0], nil
 }
 
-func (a *awsProvider) findSubnet(vpcId string) (*ec2.Subnet, error) {
+func (a *awsProvider) findSubnets(vpcId string) ([]*ec2.Subnet, error) {
 	subnets, err := a.awsClinet.DescribeSubnets(&ec2.DescribeSubnetsInput{
 		Filters: []*ec2.Filter{
 			{
@@ -345,17 +351,18 @@ func (a *awsProvider) findSubnet(vpcId string) (*ec2.Subnet, error) {
 		return nil, err
 	}
 
-	if len(subnets.Subnets) == 0 {
+	if len(subnets.Subnets) < a.gateways {
 		return nil, &errors.StatusError{
 			ErrStatus: metav1.Status{
 				Reason:  metav1.StatusReasonNotFound,
-				Message: "there are no subnets",
+				Message: fmt.Sprintf("there are no sufficient subnets (%d) for expected gateways (%d)", len(subnets.Subnets), a.gateways),
 			},
 		}
 	}
 
+	found := []*ec2.Subnet{}
 	for _, subnet := range subnets.Subnets {
-		// list the offered instance types and filters them by subnet availability zone and gateway instance type (m5n.large)
+		// list the offered instance types and filters them by subnet availability zone and gateway instance type
 		output, err := a.awsClinet.DescribeInstanceTypeOfferings(&ec2.DescribeInstanceTypeOfferingsInput{
 			LocationType: aws.String("availability-zone"),
 			Filters: []*ec2.Filter{
@@ -365,7 +372,7 @@ func (a *awsProvider) findSubnet(vpcId string) (*ec2.Subnet, error) {
 				},
 				{
 					Name:   aws.String("instance-type"),
-					Values: []*string{aws.String(instanceType)},
+					Values: []*string{aws.String(a.instanceType)},
 				},
 			},
 		})
@@ -376,11 +383,14 @@ func (a *awsProvider) findSubnet(vpcId string) (*ec2.Subnet, error) {
 		// the length of offered instance type list is not zero, it includes the gateway instance type,
 		// so we are able to create gateway instance in this subnet availability zone
 		if len(output.InstanceTypeOfferings) != 0 {
-			return subnet, nil
+			found = append(found, subnet)
+			if len(found) == a.gateways {
+				return found, nil
+			}
 		}
 	}
 
-	return nil, fmt.Errorf("the instance type %s cannot be supported in vpc %s", instanceType, vpcId)
+	return nil, fmt.Errorf("the instance type %s cannot be supported in vpc %s", a.instanceType, vpcId)
 }
 
 func (a *awsProvider) openPort(masterSecurityGroup, workerSecurityGroup *ec2.SecurityGroup, port int64, protocol string) error {
@@ -525,39 +535,46 @@ func (a *awsProvider) revokeIPsecPorts(vpc *ec2.Vpc) error {
 	return err
 }
 
-func (a *awsProvider) tagSubnet(vpc *ec2.Vpc) (*ec2.Subnet, error) {
-	subnet, err := a.findSubnet(*vpc.VpcId)
+func (a *awsProvider) tagSubnets(vpc *ec2.Vpc) ([]*ec2.Subnet, error) {
+	subnets, err := a.findSubnets(*vpc.VpcId)
 	if err != nil {
 		return nil, err
 	}
 
-	// the subnet has been labeled with internal ELB label (e.g. the submarinerconfig is reconciled again), just return it
-	for _, tag := range subnet.Tags {
-		if *tag.Key == internalELBLabel {
-			klog.V(4).Infof("subnet %s has been tagged with internal ELB label on aws", *subnet.SubnetId)
-			return subnet, nil
+	for _, subnet := range subnets {
+		// the subnet has been labeled with internal ELB label (e.g. the submarinerconfig is reconciled again), just return it
+		tagged := false
+		for _, tag := range subnet.Tags {
+			if *tag.Key == internalELBLabel {
+				klog.V(4).Infof("subnet %s has been tagged with internal ELB label on aws", *subnet.SubnetId)
+				tagged = true
+				break
+			}
+		}
+		if tagged {
+			continue
+		}
+
+		if _, err := a.awsClinet.CreateTags(&ec2.CreateTagsInput{
+			Tags: []*ec2.Tag{
+				{
+					Key:   aws.String(internalELBLabel),
+					Value: aws.String(""),
+				},
+				{
+					Key:   aws.String(internalGatewayLabel),
+					Value: aws.String(""),
+				},
+			},
+			Resources: []*string{
+				subnet.SubnetId,
+			},
+		}); err != nil {
+			return nil, err
 		}
 	}
 
-	if _, err := a.awsClinet.CreateTags(&ec2.CreateTagsInput{
-		Tags: []*ec2.Tag{
-			{
-				Key:   aws.String(internalELBLabel),
-				Value: aws.String(""),
-			},
-			{
-				Key:   aws.String(internalGatewayLabel),
-				Value: aws.String(""),
-			},
-		},
-		Resources: []*string{
-			subnet.SubnetId,
-		},
-	}); err != nil {
-		return nil, err
-	}
-
-	return subnet, nil
+	return subnets, nil
 }
 
 func (a *awsProvider) untagSubnet(vpc *ec2.Vpc) error {
@@ -615,7 +632,9 @@ func (a *awsProvider) untagSubnet(vpc *ec2.Vpc) error {
 	return operatorhelpers.NewMultiLineAggregate(errs)
 }
 
-func (a *awsProvider) deployGatewayNode(az, amiId string) error {
+func (a *awsProvider) deployGatewayNode(amiId string, subnets []*ec2.Subnet) error {
+	manifests := []workv1.Manifest{}
+
 	clusterRoleYamlData := assets.MustCreateAssetFromTemplate(
 		aggeragateClusterroleFile,
 		bindata.MustAsset(filepath.Join("", aggeragateClusterroleFile)),
@@ -624,22 +643,30 @@ func (a *awsProvider) deployGatewayNode(az, amiId string) error {
 	if err != nil {
 		return err
 	}
-	msYamlData := assets.MustCreateAssetFromTemplate(
-		manifestFile,
-		bindata.MustAsset(filepath.Join("", manifestFile)),
-		&machineSetConfig{
-			InfraId:           a.infraId,
-			AZ:                az,
-			AMIId:             amiId,
-			Region:            a.region,
-			SecurityGroupName: fmt.Sprintf("%s-submariner-gw-sg", a.infraId),
-			InstanceType:      instanceType,
-			SubnetName:        fmt.Sprintf("%s-public-%s", a.infraId, az),
-		}).Data
-	msJsonData, err := yaml.YAMLToJSON(msYamlData)
-	if err != nil {
-		return err
+	manifests = append(manifests, workv1.Manifest{RawExtension: runtime.RawExtension{Raw: clusterRoleJsonData}})
+
+	for _, subnet := range subnets {
+		az := *subnet.AvailabilityZone
+		msYamlData := assets.MustCreateAssetFromTemplate(
+			manifestFile,
+			bindata.MustAsset(filepath.Join("", manifestFile)),
+			&machineSetConfig{
+				InfraId:           a.infraId,
+				AZ:                az,
+				AMIId:             amiId,
+				Region:            a.region,
+				SecurityGroupName: fmt.Sprintf("%s-submariner-gw-sg", a.infraId),
+				InstanceType:      a.instanceType,
+				SubnetName:        fmt.Sprintf("%s-public-%s", a.infraId, az),
+				NATTPort:          a.nattPort,
+			}).Data
+		msJsonData, err := yaml.YAMLToJSON(msYamlData)
+		if err != nil {
+			return err
+		}
+		manifests = append(manifests, workv1.Manifest{RawExtension: runtime.RawExtension{Raw: msJsonData}})
 	}
+
 	return helpers.ApplyManifestWork(context.TODO(), a.workClinet, &workv1.ManifestWork{
 		TypeMeta: metav1.TypeMeta{},
 		ObjectMeta: metav1.ObjectMeta{
@@ -647,18 +674,9 @@ func (a *awsProvider) deployGatewayNode(az, amiId string) error {
 			Namespace: a.clusterName,
 		},
 		Spec: workv1.ManifestWorkSpec{
-			Workload: workv1.ManifestsTemplate{
-				Manifests: []workv1.Manifest{
-					{
-						RawExtension: runtime.RawExtension{Raw: clusterRoleJsonData},
-					},
-					{
-						RawExtension: runtime.RawExtension{Raw: msJsonData},
-					},
-				},
-			},
+			Workload: workv1.ManifestsTemplate{Manifests: manifests},
 		},
-	})
+	}, a.eventRecorder)
 }
 
 func (a *awsProvider) deleteGatewayNode() error {
